import React, { useState, useMemo, useEffect } from 'react';
import { Image, StyleSheet, Text, TouchableOpacity, View } from 'react-native';
import { CommonUtils } from '../utils';
import CloseIcon from './closeIcon';
import TimerIcon from './timerIcon';

/**
 * `Card` component represents an individual notification card in the notification list.
 *
 * @component
 * @example
 * const notification = {
 *   id: "1",
 *   message: {
 *     header: "New Message",
 *     body: "You have a new message.",
 *     avatar: { imageUrl: "https://example.com/avatar.png" },
 *     media: { thumbnail: "https://example.com/media.png" }
 *   },
 *   isRead: false,
 *   createdAt: "2020-01-01T00:00:00Z"
 * };
 *
 * <Card
 *   onCardClick={() => console.log('Notification clicked')}
 *   onDelete={() => console.log('Notification deleted')}
 *   notification={notification}
 *   cardProps={{ hideAvatar: false, showMedia: true }}
 *   styles={customStyles}
 * />
 *
 * @param {NotificationCardProps} props - The properties passed to the Card component.
 * @param {Function} props.onCardClick - Callback function executed when the card is clicked.
 * @param {Object} props.notification - The notification data to display in the card.
 * @param {Object} props.cardProps - Optional properties to customize the appearance of the card.
 * @param {Object} props.styles - Custom styles applied to the card and its elements.
 * @param {Function} props.onDelete - Callback function executed when the delete action is triggered.
 */

const Card = props => {
  const {
    onCardClick,
    notification,
    cardProps,
    styles,
    onDelete,
    darkMode
  } = props;
  const emptyState = () => {
    return darkMode ? require('../assets/emptyDark.png') : require('../assets/emptyLight.png');
  };
  const [imageSource, setImageSource] = useState(notification?.message?.avatar?.imageUrl?.length > 0 ? {
    uri: notification.message?.avatar?.imageUrl
  } : emptyState());
  useEffect(() => {
    setImageSource(notification?.message?.avatar?.imageUrl?.length > 0 ? {
      uri: notification.message?.avatar?.imageUrl
    } : emptyState());
  }, [notification, darkMode]);
  const renderAvatar = useMemo(() => {
    return /*#__PURE__*/React.createElement(View, {
      style: style.cardIconContainer
    }, /*#__PURE__*/React.createElement(View, {
      style: [style.cardIconRound, styles.cardIconRound]
    }, /*#__PURE__*/React.createElement(Image, {
      source: imageSource,
      resizeMode: "cover",
      style: style.cardAvatarStyle,
      onError: () => setImageSource(emptyState())
    })));
  }, [styles, darkMode, imageSource]);
  return /*#__PURE__*/React.createElement(TouchableOpacity, {
    onPress: () => onCardClick(notification),
    activeOpacity: 0.6,
    testID: "card-touchable",
    style: [style.cardWrapper, styles.cardWrapper, !notification?.isRead && styles.highlighted]
  }, /*#__PURE__*/React.createElement(View, {
    style: [style.activeCardMarker, styles.activeCardMarker, notification?.isRead && style.transparent]
  }), /*#__PURE__*/React.createElement(View, {
    style: [style.cardContainer, styles.cardContainer]
  }, !cardProps?.hideAvatar && renderAvatar, /*#__PURE__*/React.createElement(View, {
    style: style.cardContentContainer
  }, /*#__PURE__*/React.createElement(View, {
    style: style.cardFooterRow
  }, /*#__PURE__*/React.createElement(Text, {
    numberOfLines: 2,
    style: [styles.cardTitle, style.cardTitle]
  }, notification.message?.header), /*#__PURE__*/React.createElement(CloseIcon, {
    onDelete: onDelete,
    notification: notification,
    styles: styles
  })), Boolean(notification.message?.subHeader) && /*#__PURE__*/React.createElement(Text, {
    numberOfLines: 2,
    style: [style.cardDescription, styles.cardDescription]
  }, notification.message?.subHeader), /*#__PURE__*/React.createElement(Text, {
    numberOfLines: 2,
    style: [style.cardDescription, styles.cardDescription]
  }, notification.message?.body), /*#__PURE__*/React.createElement(View, {
    style: style.dateContainer
  }, /*#__PURE__*/React.createElement(TimerIcon, {
    styles: styles
  }), /*#__PURE__*/React.createElement(Text, {
    style: [style.dateStyle, styles.dateStyle]
  }, CommonUtils.generateElapsedTimeText(notification.createdAt))))));
};
const style = StyleSheet.create({
  cardWrapper: {
    flexDirection: 'row',
    alignItems: 'center',
    paddingRight: 2
  },
  cardContainer: {
    width: '100%',
    flexDirection: 'row',
    paddingRight: 16
  },
  cardIconContainer: {
    paddingHorizontal: 10,
    paddingTop: 4
  },
  cardTitle: {
    paddingBottom: 4,
    paddingTop: 4
  },
  icon: {
    width: '100%',
    height: '100%'
  },
  cardIconRound: {
    overflow: 'hidden',
    backgroundColor: '#D3D3D3'
  },
  cardAvatarStyle: {
    width: '100%',
    height: '100%',
    overflow: 'hidden'
  },
  cardContentContainer: {
    flex: 1,
    width: '100%',
    paddingRight: 6
  },
  cardDescription: {
    fontWeight: '400',
    paddingBottom: 10
  },
  cardFooterRow: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    alignItems: 'center'
  },
  dateStyle: {
    paddingLeft: 3
  },
  dateContainer: {
    flexDirection: 'row',
    alignItems: 'center',
    opacity: 0.8
  },
  activeCardMarker: {
    width: 4,
    height: '100%'
  },
  transparent: {
    backgroundColor: 'transparent'
  }
});
export default Card;
//# sourceMappingURL=card.js.map