import React, { useEffect, useMemo, useState, useRef } from 'react';
import { FlatList, StyleSheet, View } from 'react-native';
import PubSub from 'pubsub-js';
import Card from './card';
import EmptyWindow from './emptyWindow';
import ErrorWindow from './errorWindow';
import Header from './header';
import LoadingWindow from './loadingWindow';
import Spinner from './spinner';
import { useSirenContext } from './sirenProvider';
import { CommonUtils, Constants, useSiren } from '../utils';
const {
  DEFAULT_WINDOW_TITLE,
  ThemeMode,
  events,
  TOKEN_VERIFICATION_PENDING,
  MAXIMUM_ITEMS_PER_FETCH,
  VerificationStatus
} = Constants;
const {
  applyTheme,
  isNonEmptyArray,
  updateNotifications
} = CommonUtils;
/**
 * `SirenInbox` is a React component that displays a list of notifications fetched from the Siren SDK.
 *
 * @component
 * @example
 * const theme = {
 *   dark: { /* dark theme styles *\/ },
 *   light: { /* light theme styles *\/ }
 * };
 * <SirenInbox
 *   theme={theme}
 *   title="Notifications"
 *   hideHeader={false}
 *   darkMode={true}
 *   onError={(error) => console.log(error)}
 * />
 *
 * @param {Object} props - The props for the SirenInbox component.
 * @param {Object} [props.theme={}] - Theme object for custom styling.
 * @param {string} [props.title=DEFAULT_WINDOW_TITLE] - Title of the notification window.
 * @param {boolean} [props.hideHeader=false] - Flag to hide or show the header.
 * @param {boolean} [props.darkMode=false] - Flag to enable dark mode.
 * @param {Object} [props.cardProps={ hideAvatar: false, showMedia: true }] - Props for customizing the notification cards.
 * @param {JSX.Element} [props.listEmptyComponent=null] - Custom component to display when the notification list is empty.
 * @param {JSX.Element} [props.customHeader=null] - Custom header component.
 * @param {JSX.Element} [props.customFooter=null] - Custom footer component.
 * @param {JSX.Element} [props.customLoader=null] - Custom loader component.
 * @param {JSX.Element} [props.customErrorWindow=null] - Custom error component.
 * @param {Function} [props.customNotificationCard=null] - Custom function for rendering notification cards.
 * @param {Function} [props.onNotificationCardClick=() => null] - Callback for handling notification card clicks.
 * @param {Function} [props.onError] - Callback for handling errors.
 */
const SirenInbox = props => {
  const {
    theme = {
      dark: {},
      light: {}
    },
    customStyles = {},
    title = DEFAULT_WINDOW_TITLE,
    hideHeader = false,
    darkMode = false,
    cardProps = {
      hideAvatar: false,
      showMedia: true
    },
    listEmptyComponent = null,
    customHeader = null,
    customFooter = null,
    customLoader = null,
    customErrorWindow = null,
    customNotificationCard = null,
    onNotificationCardClick = () => null,
    onError = () => {},
    hideClearAll = false,
    itemsPerFetch = 20
  } = props;
  const notificationsPerPage = itemsPerFetch > MAXIMUM_ITEMS_PER_FETCH ? MAXIMUM_ITEMS_PER_FETCH : itemsPerFetch;
  const {
    siren,
    verificationStatus
  } = useSirenContext();
  const {
    deleteNotification,
    deleteNotificationsByDate,
    markNotificationsAsViewed
  } = useSiren();
  const [notifications, setNotifications] = useState([]);
  const [isLoading, setIsLoading] = useState(true);
  const [endReached, setEndReached] = useState(false);
  const [isError, setIsError] = useState(false);
  const [eventListenerData, setEventListenerData] = useState(null);
  const disableCardDelete = useRef(false);
  useEffect(() => {
    PubSub.subscribe(events.NOTIFICATION_LIST_EVENT, notificationSubscriber);
    return cleanUp();
  }, []);
  useEffect(() => {
    // Initialize Siren SDK and start polling notifications
    if (verificationStatus !== VerificationStatus.PENDING && siren) initialize();
  }, [siren, verificationStatus]);
  useEffect(() => {
    if (eventListenerData) {
      const updatedNotifications = updateNotifications(eventListenerData, notifications);
      setNotifications(updatedNotifications);
      setEventListenerData(null);
    }
  }, [eventListenerData]);
  const handleMarkNotificationsAsViewed = async (newNotifications = notifications) => {
    if (isNonEmptyArray(newNotifications)) {
      const response = await markNotificationsAsViewed(newNotifications[0].createdAt);
      processError(response?.error);
    }
  };
  const processError = error => {
    if (error) {
      if (error?.Code !== TOKEN_VERIFICATION_PENDING) setIsError(true);
      if (onError) onError(error);
    }
  };

  // Clean up - stop polling when component unmounts
  const cleanUp = () => () => {
    siren?.stopRealTimeNotificationFetch();
    setNotifications([]);
    PubSub.unsubscribe(events.NOTIFICATION_LIST_EVENT);
    handleMarkNotificationsAsViewed();
  };
  const notificationSubscriber = async (type, dataString) => {
    const data = await JSON.parse(dataString);
    setEventListenerData(data);
  };

  // Initialize Siren SDK and fetch notifications
  const initialize = async () => {
    if (siren) {
      siren?.stopRealTimeNotificationFetch();
      const allNotifications = await fetchNotifications(siren, true);
      const notificationParams = {
        size: notificationsPerPage
      };
      if (isNonEmptyArray(allNotifications)) notificationParams.start = allNotifications[0].createdAt;
      if (verificationStatus === VerificationStatus.SUCCESS) siren?.startRealTimeNotificationFetch(notificationParams);
    }
  };
  const generateNotificationParams = attachEndDate => {
    const notificationParams = {
      size: notificationsPerPage,
      sort: 'createdAt'
    };
    if (attachEndDate) notificationParams.end = notifications[notifications.length - 1].createdAt;
    return notificationParams;
  };
  const processResponse = (nonEmptyResponse, isResetList, responseData) => {
    if (nonEmptyResponse) {
      const updatedNotifications = isResetList ? responseData : [...notifications, ...responseData];
      isResetList && handleMarkNotificationsAsViewed(updatedNotifications);
      setNotifications(updatedNotifications);
      return updatedNotifications;
    } else {
      setEndReached(true);
    }
    return [];
  };

  // Fetch notifications
  const fetchNotifications = async (siren, isResetList = false) => {
    setIsError(false);
    setIsLoading(true);
    let updatedNotifications = notifications;
    if (siren) try {
      const notificationParams = generateNotificationParams(!isResetList);
      const response = await siren.fetchAllNotifications(notificationParams);
      const nonEmptyResponse = Boolean(isNonEmptyArray(response?.data));
      if (response?.data) updatedNotifications = processResponse(nonEmptyResponse, isResetList, response.data);
      if (response?.error) processError(response.error);
      setIsLoading(false);
    } catch (err) {
      setIsLoading(false);
      setIsError(true);
    }
    return updatedNotifications;
  };

  // Apply theme styles
  const styles = useMemo(() => applyTheme(darkMode ? theme?.dark : theme?.light, customStyles, darkMode ? ThemeMode.DARK : ThemeMode.LIGHT), [theme, darkMode, customStyles]);

  // Refresh notifications
  const onRefresh = async () => {
    if (siren) try {
      setEndReached(false);
      setIsError(false);
      setNotifications([]);
      setIsLoading(true);
      siren?.stopRealTimeNotificationFetch();
      const allNotifications = (await fetchNotifications(siren, true)) || [];
      const notificationParams = {
        size: notificationsPerPage
      };
      if (isNonEmptyArray(allNotifications)) notificationParams.start = allNotifications[0].createdAt;
      if (verificationStatus === VerificationStatus.SUCCESS) siren?.startRealTimeNotificationFetch(notificationParams);
    } catch (err) {
      setIsLoading(false);
      setIsError(true);
    }
  };

  // Load more notifications when reaching end of list
  const onEndReached = () => {
    const fetchMore = siren && !isLoading && !endReached && isNonEmptyArray(notifications);
    if (fetchMore) fetchNotifications(siren, false);
  };

  // Render empty window, error window, or custom empty component
  const renderListEmpty = () => {
    if (!isLoading) {
      if (isError) return /*#__PURE__*/React.createElement(ErrorWindow, {
        styles: styles,
        darkMode: darkMode,
        customErrorWindow: customErrorWindow
      });
      return listEmptyComponent || /*#__PURE__*/React.createElement(EmptyWindow, {
        styles: styles,
        darkMode: darkMode
      });
    }
    return /*#__PURE__*/React.createElement(LoadingWindow, {
      styles: styles,
      customLoader: customLoader
    });
  };
  const onDelete = async id => {
    if (!disableCardDelete.current) {
      disableCardDelete.current = true;
      const response = await deleteNotification(id);
      processError(response?.error);
      disableCardDelete.current = false;
    }
  };
  const onPressClearAll = async () => {
    if (isNonEmptyArray(notifications)) {
      const response = await deleteNotificationsByDate(notifications[0].createdAt);
      if (response?.error) {
        processError(response?.error);
      } else {
        setIsLoading(false);
        setNotifications([]);
        setEndReached(false);
      }
    }
  };
  const renderDefaultNotificationCard = item => {
    return /*#__PURE__*/React.createElement(Card, {
      onCardClick: onNotificationCardClick,
      notification: item,
      cardProps: cardProps,
      styles: styles,
      onDelete: onDelete,
      darkMode: darkMode
    });
  };

  // Render notification card
  const renderCard = ({
    item
  }) => {
    if (customNotificationCard) return customNotificationCard(item);
    return renderDefaultNotificationCard(item);
  };

  // Render loading window
  const renderListFooter = () => {
    if (isLoading && isNonEmptyArray(notifications)) return /*#__PURE__*/React.createElement(Spinner, {
      theme: darkMode ? theme?.dark : theme?.light,
      mode: darkMode ? ThemeMode.DARK : ThemeMode.LIGHT
    });
    return null;
  };
  const renderHeader = () => {
    if (!hideHeader) {
      if (customHeader) return customHeader;
      return /*#__PURE__*/React.createElement(Header, {
        title: title,
        styles: styles,
        onPressClearAll: onPressClearAll,
        hideClearAll: hideClearAll,
        clearAllDisabled: !isNonEmptyArray(notifications)
      });
    }
    return null;
  };
  const renderList = () => {
    return /*#__PURE__*/React.createElement(FlatList, {
      data: notifications,
      renderItem: renderCard,
      keyExtractor: item => item.id,
      onRefresh: onRefresh,
      refreshing: false,
      contentContainerStyle: styles.contentContainer,
      onEndReached: onEndReached,
      ListFooterComponent: renderListFooter
    });
  };
  return /*#__PURE__*/React.createElement(View, {
    style: [style.container, styles.container]
  }, renderHeader(), isNonEmptyArray(notifications) ? renderList() : renderListEmpty(), customFooter || null);
};
const style = StyleSheet.create({
  container: {
    minWidth: 300,
    flex: 1
  }
});
export default SirenInbox;
//# sourceMappingURL=sirenInbox.js.map